import {
  JsonEncoder
} from "./chunk-VBSEGKT7.js";
import {
  uname,
  uuid
} from "./chunk-BMLTMNZA.js";
import {
  DefaultLogger
} from "./chunk-2DAGZ4FT.js";
import {
  isPromise,
  tryTimeout
} from "./chunk-L324TYGN.js";
import {
  valueToString
} from "./chunk-QUQRUDWG.js";

// src/common/msg/messages.ts
function createPromiseProxy(fn, opt, predefinedMethods = {}) {
  return new Proxy(predefinedMethods, {
    get: (target, name) => {
      if (name in target)
        return target[name];
      return (...args) => fn(name, args, opt);
    }
  });
}
function useMessageHub(opt = {}) {
  const {
    name = uname("hub"),
    encoder = new JsonEncoder(),
    retryAfter = 1e3,
    ignoreUnhandled = true,
    logLevel = false
  } = opt;
  const log = DefaultLogger(name, logLevel);
  const handlers = {};
  let channel;
  const queue = [];
  let queueRetryTimer;
  const waitingForResponse = {};
  const dispose = () => {
    clearTimeout(queueRetryTimer);
  };
  const postNext = async () => {
    clearTimeout(queueRetryTimer);
    if (channel) {
      if (channel.isConnected) {
        while (queue.length) {
          const message = queue[0];
          try {
            channel.postMessage(await encoder.encode(message));
            queue.shift();
          } catch (err) {
            log.warn("postMessage", err);
            break;
          }
        }
      }
      if (queue.length > 0 && retryAfter > 0)
        queueRetryTimer = setTimeout(postNext, retryAfter);
    }
  };
  const postMessage = async (message) => {
    log("enqueue postMessage", message);
    queue.push(message);
    await postNext();
  };
  const connect = async (newChannel) => {
    channel = newChannel;
    channel.on("connect", postNext);
    channel.on("message", async (msg) => {
      log("onmessage", typeof msg);
      const { name: name2, args, id, result, error } = await encoder.decode(msg.data);
      if (name2) {
        log(`name ${name2} id ${id}`);
        try {
          if (handlers[name2] == null)
            throw new Error(`handler for ${name2} was not found`);
          let result2 = handlers[name2](...args);
          if (isPromise(result2))
            result2 = await result2;
          log(`result ${result2}`);
          if (id)
            await postMessage({ id, result: result2 });
        } catch (error2) {
          const err = error2 instanceof Error ? error2 : new Error(valueToString(error2));
          log.warn("execution error", err.name);
          await postMessage({
            id,
            error: {
              message: err.message,
              stack: err.stack,
              name: err.name
            }
          });
        }
      } else if (id) {
        log(`response for id=${id}: result=${result}, error=${error}`);
        if (waitingForResponse[id] == null) {
          if (result === void 0)
            log(`skip response for ${id}`);
          else
            log.warn(`no response hook for ${id}`);
        } else {
          const [resolve, reject] = waitingForResponse[id];
          if (resolve && reject) {
            delete waitingForResponse[id];
            if (error) {
              const err = new Error(error.message);
              err.stack = error.stack;
              err.name = error.name;
              log.warn("reject", err.name);
              reject(err);
            } else {
              log("resolve", result);
              resolve(result);
            }
          }
        }
      } else if (!ignoreUnhandled) {
        log.warn("Unhandled message", msg);
      }
    });
    await postNext();
  };
  const fetchMessage = async (name2, args, opt2 = {}) => {
    const { timeout = 5e3 } = opt2;
    const id = uuid();
    await postMessage({
      name: name2,
      args,
      id
    });
    return tryTimeout(
      new Promise(
        (resolve, reject) => waitingForResponse[id] = [resolve, reject]
      ),
      timeout
    );
  };
  if (opt.channel)
    void connect(opt.channel);
  return {
    dispose,
    connect,
    listen(newHandlers) {
      Object.assign(handlers, newHandlers);
    },
    send() {
      return createPromiseProxy(fetchMessage, {}, {
        options(perCallopt) {
          return createPromiseProxy(fetchMessage, { ...perCallopt });
        }
      });
    }
  };
}

export {
  createPromiseProxy,
  useMessageHub
};
//# sourceMappingURL=chunk-A4L4FO52.js.map