import {
  getUtf8TextDecoder
} from "./chunk-GKKLBSFA.js";
import {
  BIT7,
  BIT8,
  BITS6,
  BITS7
} from "./chunk-RNL46FQ3.js";
import {
  createUint8ArrayViewFromArrayBuffer
} from "./chunk-MWUFUBY5.js";

// src/common/bin/lib0/decoding.ts
var errorUnexpectedEndOfArray = "Unexpected end of array";
var errorIntegerOutOfRange = "Integer out of Range";
var BinDecoder = class {
  constructor(uint8Array) {
    this.arr = uint8Array;
    this.pos = 0;
  }
};
function createDecoder(uint8Array) {
  return new BinDecoder(uint8Array);
}
function hasContent(decoder) {
  return decoder.pos !== decoder.arr.length;
}
function clone(decoder, newPos = decoder.pos) {
  const _decoder = createDecoder(decoder.arr);
  _decoder.pos = newPos;
  return _decoder;
}
function readUint8Array(decoder, len) {
  const view = createUint8ArrayViewFromArrayBuffer(decoder.arr.buffer, decoder.pos + decoder.arr.byteOffset, len);
  decoder.pos += len;
  return view;
}
function readVarUint(decoder) {
  let num = 0;
  let mult = 1;
  const len = decoder.arr.length;
  while (decoder.pos < len) {
    const r = decoder.arr[decoder.pos++];
    num = num + (r & BITS7) * mult;
    mult *= 128;
    if (r < BIT8)
      return num;
    if (num > Number.MAX_SAFE_INTEGER)
      throw new Error(errorIntegerOutOfRange);
  }
  throw new Error(errorUnexpectedEndOfArray);
}
function readVarUint8Array(decoder) {
  return readUint8Array(decoder, readVarUint(decoder));
}
function readTailAsUint8Array(decoder) {
  return readUint8Array(decoder, decoder.arr.length - decoder.pos);
}
function skip8(decoder) {
  return decoder.pos++;
}
function readUint8(decoder) {
  return decoder.arr[decoder.pos++];
}
function readUint16(decoder) {
  const uint = decoder.arr[decoder.pos] + (decoder.arr[decoder.pos + 1] << 8);
  decoder.pos += 2;
  return uint;
}
function readUint32(decoder) {
  const uint = decoder.arr[decoder.pos] + (decoder.arr[decoder.pos + 1] << 8) + (decoder.arr[decoder.pos + 2] << 16) + (decoder.arr[decoder.pos + 3] << 24) >>> 0;
  decoder.pos += 4;
  return uint;
}
function readUint32BigEndian(decoder) {
  const uint = decoder.arr[decoder.pos + 3] + (decoder.arr[decoder.pos + 2] << 8) + (decoder.arr[decoder.pos + 1] << 16) + (decoder.arr[decoder.pos] << 24) >>> 0;
  decoder.pos += 4;
  return uint;
}
function peekUint8(decoder) {
  return decoder.arr[decoder.pos];
}
function peekUint16(decoder) {
  return decoder.arr[decoder.pos] + (decoder.arr[decoder.pos + 1] << 8);
}
function peekUint32(decoder) {
  return decoder.arr[decoder.pos] + (decoder.arr[decoder.pos + 1] << 8) + (decoder.arr[decoder.pos + 2] << 16) + (decoder.arr[decoder.pos + 3] << 24) >>> 0;
}
function readVarInt(decoder) {
  let r = decoder.arr[decoder.pos++];
  let num = r & BITS6;
  let mult = 64;
  const sign = (r & BIT7) > 0 ? -1 : 1;
  if ((r & BIT8) === 0) {
    return sign * num;
  }
  const len = decoder.arr.length;
  while (decoder.pos < len) {
    r = decoder.arr[decoder.pos++];
    num = num + (r & BITS7) * mult;
    mult *= 128;
    if (r < BIT8)
      return sign * num;
    if (num > Number.MAX_SAFE_INTEGER)
      throw new Error(errorIntegerOutOfRange);
  }
  throw new Error(errorUnexpectedEndOfArray);
}
function peekVarUint(decoder) {
  const pos = decoder.pos;
  const s = readVarUint(decoder);
  decoder.pos = pos;
  return s;
}
function peekVarInt(decoder) {
  const pos = decoder.pos;
  const s = readVarInt(decoder);
  decoder.pos = pos;
  return s;
}
function _readVarStringPolyfill(decoder) {
  let remainingLen = readVarUint(decoder);
  if (remainingLen === 0) {
    return "";
  } else {
    let encodedString = String.fromCodePoint(readUint8(decoder));
    if (--remainingLen < 100) {
      while (remainingLen--)
        encodedString += String.fromCodePoint(readUint8(decoder));
    } else {
      while (remainingLen > 0) {
        const nextLen = remainingLen < 1e4 ? remainingLen : 1e4;
        const bytes = decoder.arr.subarray(decoder.pos, decoder.pos + nextLen);
        decoder.pos += nextLen;
        encodedString += String.fromCodePoint.apply(null, bytes);
        remainingLen -= nextLen;
      }
    }
    return decodeURIComponent(escape(encodedString));
  }
}
function readVarString(decoder) {
  const utf8TextDecoder = getUtf8TextDecoder();
  return utf8TextDecoder ? utf8TextDecoder.decode(readVarUint8Array(decoder)) : _readVarStringPolyfill(decoder);
}
function peekVarString(decoder) {
  const pos = decoder.pos;
  const s = readVarString(decoder);
  decoder.pos = pos;
  return s;
}
function readFromDataView(decoder, len) {
  const dv = new DataView(decoder.arr.buffer, decoder.arr.byteOffset + decoder.pos, len);
  decoder.pos += len;
  return dv;
}
function readFloat32(decoder) {
  return readFromDataView(decoder, 4).getFloat32(0, false);
}
function readFloat64(decoder) {
  return readFromDataView(decoder, 8).getFloat64(0, false);
}
function readBigInt64(decoder) {
  return readFromDataView(decoder, 8).getBigInt64(0, false);
}
function readBigUint64(decoder) {
  return readFromDataView(decoder, 8).getBigUint64(0, false);
}
var readAnyLookupTable = [
  (_) => void 0,
  // CASE 127: undefined
  (_) => null,
  // CASE 126: null
  readVarInt,
  // CASE 125: integer
  readFloat32,
  // CASE 124: float32
  readFloat64,
  // CASE 123: float64
  readBigInt64,
  // CASE 122: bigint
  (_) => false,
  // CASE 121: boolean (false)
  (_) => true,
  // CASE 120: boolean (true)
  readVarString,
  // CASE 119: string
  (decoder) => {
    const len = readVarUint(decoder);
    const obj = {};
    for (let i = 0; i < len; i++) {
      const key = readVarString(decoder);
      obj[key] = readAny(decoder);
    }
    return obj;
  },
  (decoder) => {
    const len = readVarUint(decoder);
    const arr = [];
    for (let i = 0; i < len; i++)
      arr.push(readAny(decoder));
    return arr;
  },
  readVarUint8Array
  // CASE 116: Uint8Array
];
function readAny(decoder) {
  return readAnyLookupTable[127 - readUint8(decoder)](decoder);
}

export {
  BinDecoder,
  createDecoder,
  hasContent,
  clone,
  readUint8Array,
  readVarUint,
  readVarUint8Array,
  readTailAsUint8Array,
  skip8,
  readUint8,
  readUint16,
  readUint32,
  readUint32BigEndian,
  peekUint8,
  peekUint16,
  peekUint32,
  readVarInt,
  peekVarUint,
  peekVarInt,
  readVarString,
  peekVarString,
  readFromDataView,
  readFloat32,
  readFloat64,
  readBigInt64,
  readBigUint64,
  readAny
};
//# sourceMappingURL=chunk-DVBRAJ3X.js.map